@@ -1,5 +1,5 @@
 ;;; gnus-msg.el --- mail and post interface for Semi-gnus
-;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002
+;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003
 ;;        Free Software Foundation, Inc.
 
 ;; Author: Masanobu UMEDA <umerin@flab.flab.fujitsu.junet>
@@ -120,6 +120,7 @@ the second with the current group name."
   "*Alist of styles to use when posting.
 See Info node `(gnus)Posting Styles'."
   :group 'gnus-message
+  :link '(custom-manual "(gnus)Posting Styles")
   :type '(repeat (cons (choice (regexp)
 			       (variable)
 			       (list (const header)
@@ -179,7 +180,7 @@ See Info node `(gnus)Posting Styles'."
   "Should local-file attachments be included as external parts in Gcc copies?
 If it is `all', attach files as external parts;
 if a regexp and matches the Gcc group name, attach files as external parts;
-If nil, attach files as normal parts."
+if nil, attach files as normal parts."
   :version "21.1"
   :group 'gnus-message
   :type '(choice (const nil :tag "None")
@@ -281,7 +282,23 @@ See also the `mml-default-encrypt-method' variable."
 (defcustom gnus-confirm-mail-reply-to-news nil
   "If non-nil, Gnus requests confirmation when replying to news.
 This is done because new users often reply by mistake when reading
-news."
+news.
+This can also be a function receiving the group name as the only
+parameter which should return non-nil iff a confirmation is needed, or
+a regexp, in which case a confirmation is asked for iff the group name
+matches the regexp."
+  :group 'gnus-message
+  :type '(choice (const :tag "No" nil)
+		 (const :tag "Yes" t)
+		 (regexp :tag "Iff group matches regexp")
+		 (function :tag "Iff function evaluates to non-nil")))
+
+(defcustom gnus-confirm-treat-mail-like-news
+  nil
+  "If non-nil, Gnus will treat mail like news with regard to confirmation
+when replying by mail.  See the `gnus-confirm-mail-reply-to-news' variable
+for fine-tuning this.
+If nil, Gnus will never ask for confirmation if replying to mail."
   :group 'gnus-message
   :type 'boolean)
 
@@ -297,6 +314,7 @@ If nil, the address field will always be empty after invoking
 (defvar gnus-inhibit-posting-styles nil
   "Inhibit the use of posting styles.")
 
+(defvar gnus-article-yanked-articles nil)
 (defvar gnus-message-buffer "*Mail Gnus*")
 (defvar gnus-article-copy nil)
 (defvar gnus-check-before-posting nil)
@@ -383,15 +401,22 @@ Thank you for your help in stamping out bugs.
 
 ;;; Internal functions.
 
+(defun gnus-inews-make-draft ()
+  `(lambda ()
+     (gnus-inews-make-draft-meta-information
+      ,gnus-newsgroup-name ',gnus-article-reply)))
+
 (defvar gnus-article-reply nil)
 (defmacro gnus-setup-message (config &rest forms)
   (let ((winconf (make-symbol "gnus-setup-message-winconf"))
 	(buffer (make-symbol "gnus-setup-message-buffer"))
 	(article (make-symbol "gnus-setup-message-article"))
+	(yanked (make-symbol "gnus-setup-yanked-articles"))
 	(group (make-symbol "gnus-setup-message-group")))
     `(let ((,winconf (current-window-configuration))
 	   (,buffer (buffer-name (current-buffer)))
 	   (,article gnus-article-reply)
+	   (,yanked gnus-article-yanked-articles)
 	   (,group gnus-newsgroup-name)
 	   (message-header-setup-hook
 	    (copy-sequence message-header-setup-hook))
@@ -412,11 +437,19 @@ Thank you for your help in stamping out bugs.
        (add-hook 'message-mode-hook
 		 (lambda ()
 		   (gnus-configure-posting-styles ,group)))
+       (gnus-pull ',(intern gnus-draft-meta-information-header)
+		  message-required-headers)
+       (when (and ,group
+		  (not (string= ,group "")))
+	 (push (cons
+		(intern gnus-draft-meta-information-header)
+		(gnus-inews-make-draft))
+	       message-required-headers))
        (unwind-protect
 	   (progn
 	     ,@forms)
-	 (gnus-inews-add-send-actions ,winconf ,buffer ,article ,config)
-	 (gnus-inews-insert-draft-meta-information ,group ,article)
+	 (gnus-inews-add-send-actions ,winconf ,buffer ,article ,config
+				      ,yanked)
 	 (setq gnus-message-buffer (current-buffer))
 	 (set (make-local-variable 'gnus-message-group-art)
 	      (cons ,group ,article))
@@ -428,18 +461,13 @@ Thank you for your help in stamping out bugs.
        (run-hooks 'post-command-hook)
        (set-buffer-modified-p nil))))
 
-(defun gnus-inews-insert-draft-meta-information (group article)
-  (save-excursion
-    (when (and group
-	       (not (string= group ""))
-	       (not (message-fetch-field gnus-draft-meta-information-header)))
-      (goto-char (point-min))
-      (insert gnus-draft-meta-information-header ": (\"" group "\" "
-	      (if article (number-to-string
-			   (if (listp article)
-			       (car article)
-			     article)) "\"\"")
-	      ")\n"))))
+(defun gnus-inews-make-draft-meta-information (group article)
+  (concat "(\"" group "\" "
+	  (if article (number-to-string
+		       (if (listp article)
+			   (car article)
+			 article)) "\"\"")
+	  ")"))
 
 ;;;###autoload
 (defun gnus-msg-mail (&optional to subject other-headers continue
@@ -502,7 +530,8 @@ Gcc: header for archiving purposes."
 			 (symbol-value (car elem))))
 	    (throw 'found (cons (cadr elem) (caddr elem)))))))))
 
-(defun gnus-inews-add-send-actions (winconf buffer article &optional config)
+(defun gnus-inews-add-send-actions (winconf buffer article
+					    &optional config yanked)
   (make-local-hook 'message-sent-hook)
   (add-hook 'message-sent-hook (if gnus-agent 'gnus-agent-possibly-do-gcc
 				 'gnus-inews-do-gcc) nil t)
@@ -516,15 +545,19 @@ Gcc: header for archiving purposes."
   (when (not message-use-multi-frames)
     (message-add-action
      `(set-window-configuration ,winconf) 'exit 'postpone 'kill))
-  (message-add-action
-   `(when (gnus-buffer-exists-p ,buffer)
-      (save-excursion
-	(set-buffer ,buffer)
-	,(when article
-	   (if (eq config 'forward)
-	       `(gnus-summary-mark-article-as-forwarded ',article)
-	     `(gnus-summary-mark-article-as-replied ',article)))))
-   'send))
+  (let ((to-be-marked (cond
+		       (yanked yanked)
+		       (article (if (listp article) article (list article)))
+		       (t nil))))
+    (message-add-action
+     `(when (gnus-buffer-exists-p ,buffer)
+	(save-excursion
+	  (set-buffer ,buffer)
+	  ,(when to-be-marked
+	     (if (eq config 'forward)
+		 `(gnus-summary-mark-article-as-forwarded ',to-be-marked)
+	       `(gnus-summary-mark-article-as-replied ',to-be-marked)))))
+     'send)))
 
 (put 'gnus-setup-message 'lisp-indent-function 1)
 (put 'gnus-setup-message 'edebug-form-spec '(form body))
@@ -606,7 +639,8 @@ a news."
 	   ""))
 	;; make sure last viewed article doesn't affect posting styles:
 	(gnus-article-copy))
-    (gnus-post-news 'post gnus-newsgroup-name)))
+    (gnus-post-news 'post gnus-newsgroup-name nil nil nil nil
+		    (string= gnus-newsgroup-name ""))))
 
 (defun gnus-summary-mail-other-window (&optional arg)
   "Start composing a mail in another window.
@@ -761,8 +795,7 @@ yanked."
 	     (with-current-buffer gnus-article-copy
 	       (save-restriction
 		 (nnheader-narrow-to-headers)
-		 (ietf-drums-unfold-fws)
-		 (nnheader-parse-head t)))))
+		 (nnheader-parse-naked-head)))))
 	(message-yank-original)
 	(setq beg (or beg (mark t))))
       (when articles
@@ -892,12 +925,14 @@ header line with the old Message-ID."
 	    (goto-char (point-min))
 	    (while (looking-at message-unix-mail-delimiter)
 	      (forward-line 1))
-	    (setq beg (point)
-		  end (or (message-goto-body) beg))
+	    (let ((mail-header-separator ""))
+	      (setq beg (point)
+		    end (or (message-goto-body) beg)))
 	    ;; Delete the headers from the displayed articles.
 	    (set-buffer gnus-article-copy)
-	    (delete-region (goto-char (point-min))
-			   (or (message-goto-body) (point-max)))
+	    (let ((mail-header-separator ""))
+	      (delete-region (goto-char (point-min))
+			     (or (message-goto-body) (point-max))))
 	    ;; Insert the original article headers.
 	    (insert-buffer-substring gnus-original-article-buffer beg end)
 	    ;; Decode charsets.
@@ -915,6 +950,7 @@ header line with the old Message-ID."
   (when article-buffer
     (gnus-copy-article-buffer))
   (let ((gnus-article-reply (and article-buffer (gnus-summary-article-number)))
+	(gnus-article-yanked-articles yank)
 	(add-to-list gnus-add-to-list))
     (gnus-setup-message (cond (yank 'reply-yank)
 			      (article-buffer 'reply)
@@ -973,7 +1009,7 @@ header line with the old Message-ID."
 	  (gnus-inews-yank-articles yank))))))
 
 (defun gnus-msg-treat-broken-reply-to (&optional force)
-  "Remove the Reply-to header iff broken-reply-to."
+  "Remove the Reply-to header if broken-reply-to."
   (when (or force
 	    (gnus-group-find-parameter
 	     gnus-newsgroup-name 'broken-reply-to))
@@ -1162,15 +1198,23 @@ If VERY-WIDE, make a very wide reply."
    (list (and current-prefix-arg
 	      (gnus-summary-work-articles 1))))
   ;; Allow user to require confirmation before replying by mail to the
-  ;; author of a news article.
-  (when (or (not (gnus-news-group-p gnus-newsgroup-name))
-	    (not gnus-confirm-mail-reply-to-news)
+  ;; author of a news article (or mail message).
+  (when (or 
+	    (not (or (gnus-news-group-p gnus-newsgroup-name)
+		     gnus-confirm-treat-mail-like-news))
+	    (not (cond ((stringp gnus-confirm-mail-reply-to-news)
+			(string-match gnus-confirm-mail-reply-to-news
+				      gnus-newsgroup-name))
+		       ((functionp gnus-confirm-mail-reply-to-news)
+			(funcall gnus-confirm-mail-reply-to-news gnus-newsgroup-name))
+		       (t gnus-confirm-mail-reply-to-news)))
 	    (y-or-n-p "Really reply by mail to article author? "))
     (let* ((article
 	    (if (listp (car yank))
 		(caar yank)
 	      (car yank)))
 	   (gnus-article-reply (or article (gnus-summary-article-number)))
+	   (gnus-article-yanked-articles yank)
 	   (headers ""))
       ;; Stripping headers should be specified with mail-yank-ignored-headers.
       (when yank
@@ -1278,17 +1322,22 @@ If FULL-HEADERS (the prefix), include full headers when forwarding.
 Note that this function definition for T-gnus is totally different
 from the original Gnus."
   (interactive "P")
-  (if (null (cdr (gnus-summary-work-articles nil)))
+  (if (cdr (gnus-summary-work-articles nil))
+      ;; Process marks are given.
+      (gnus-summary-digest-mail-forward nil post)
+    ;; No process marks.
+    (let* ((gnus-article-reply (gnus-summary-article-number))
+	   (gnus-article-yanked-articles (list gnus-article-reply))
+	   charset
+	   (message-included-forward-headers
+	    (if full-headers "" message-included-forward-headers)))
       (gnus-setup-message 'forward
 	(gnus-summary-select-article)
-	(let ((charset default-mime-charset))
-	  (set-buffer gnus-original-article-buffer)
-	  (make-local-variable 'default-mime-charset)
-	  (setq default-mime-charset charset))
-	(let ((message-included-forward-headers
-	       (if full-headers "" message-included-forward-headers)))
-	  (message-forward post)))
-    (gnus-summary-digest-mail-forward nil post)))
+	(setq charset default-mime-charset)
+	(set-buffer gnus-original-article-buffer)
+	(make-local-variable 'default-mime-charset)
+	(setq default-mime-charset charset)
+	(message-forward post)))))
 
 (defun gnus-summary-digest-mail-forward (&optional n post)
   "Digests and forwards all articles in this series.
@@ -1298,9 +1347,10 @@ If N is nil and any articles have been marked with the process mark,
 forward those articles instead.
 Optional POST will use news to forward instead of mail."
   (interactive "P")
-  (let ((articles (gnus-summary-work-articles n))
-	(topics "Topics:\n")
-	subject article frame)
+  (let* ((articles (gnus-summary-work-articles n))
+	 (gnus-article-yanked-articles (copy-sequence articles))
+	 (topics "Topics:\n")
+	 subject article frame)
     (when (car articles)
       (gnus-setup-message 'forward
 	(gnus-summary-select-article)
@@ -1683,17 +1733,15 @@ The source file has to be in the Emacs load path."
       (insert ";----------------- Environment follows ------------------\n\n"))
     (while olist
       (if (boundp (car olist))
-	  (condition-case ()
-	      (pp `(setq ,(car olist)
-			 ,(if (or (consp (setq sym (symbol-value (car olist))))
-				  (and (symbolp sym)
-				       (not (or (eq sym nil)
-						(eq sym t)))))
-			      (list 'quote (symbol-value (car olist)))
-			    (symbol-value (car olist))))
-		  (current-buffer))
-	    (error
-	     (format "(setq %s 'whatever)\n" (car olist))))
+	  (ignore-errors
+	    (pp `(setq ,(car olist)
+		       ,(if (or (consp (setq sym (symbol-value (car olist))))
+				(and (symbolp sym)
+				     (not (or (eq sym nil)
+					      (eq sym t)))))
+			    (list 'quote (symbol-value (car olist)))
+			  (symbol-value (car olist))))
+		(current-buffer)))
 	(insert ";; (makeunbound '" (symbol-name (car olist)) ")\n"))
       (setq olist (cdr olist)))
     ;; Remove any control chars - they seem to cause trouble for some
@@ -1789,6 +1837,9 @@ this is a reply."
 			      group (gnus-status-message method))
 		(sit-for 2))
 	      (when (and group-art
+			 ;; FIXME: Should gcc-mark-as-read work when
+			 ;; Gnus is not running?
+			 (gnus-alive-p)
 			 (or gnus-gcc-mark-as-read
 			     gnus-inews-mark-gcc-as-read))
 		(gnus-group-mark-article-read group (cdr group-art)))
@@ -1807,7 +1858,7 @@ this is a reply."
 		    group))))
 	(when gcc
 	  (insert "Gcc: "
-		  (if (stringp gcc) 
+		  (if (stringp gcc)
 		      (if (string-match " " gcc)
 			  (concat "\"" gcc "\"")
 			gcc)
@@ -1953,7 +2004,7 @@ this is a reply."
 		  ;; This is a form to be evaled.
 		  (eval match)))))
 	  ;; We have a match, so we set the variables.
-	  (setq style (gnus-configure-posting-style style))
+	  (setq style (gnus-configure-posting-style style nil))
 	  (dolist (attribute style)
 	    (setq results (delq (assoc (car attribute) results) results))
 	    (push attribute results))))
@@ -1963,6 +2014,8 @@ this is a reply."
       (setq results (delq name (delq address results)))
       ;; make-local-hook is not obsolete in Emacs 20 or XEmacs.
       (make-local-hook 'message-setup-hook)
+      (setq results (sort results (lambda (x y)
+				    (string-lessp (car x) (car y)))))
       (dolist (result results)
 	(add-hook 'message-setup-hook
 		  (cond
@@ -1994,7 +2047,9 @@ this is a reply."
 			   (let ((value ,(cdr result)))
 			     (when value
 			       (message-goto-eoh)
-			       (insert ,header ": " value "\n"))))))))
+			       (insert ,header ": " value)
+			       (unless (bolp)
+				 (insert "\n")))))))))
 		  nil 'local))
       (when (or name address)
 	(add-hook 'message-setup-hook
@@ -2011,7 +2066,7 @@ this is a reply."
 		  nil 'local)))))
 
 ;; splitted from gnus-configure-posting-styles to allow recursive traversal.
-(defun gnus-configure-posting-style (style)
+(defun gnus-configure-posting-style (style stack)
   "Parse one posting style STYLE and returns the value as an alist."
   (let (results element variable filep value v)
     (dolist (attribute style)
@@ -2057,10 +2112,14 @@ this is a reply."
 		    (delete-char -1))
 		  (buffer-string))))
       (if (eq element 'import)
-	  (setq results
-		(nconc (nreverse (gnus-configure-posting-style
-				  (cdr (assoc v gnus-named-posting-styles))))
-		       results))
+	  (progn
+	    (if (member v stack)
+		(error "Circular import of \"%s\"" v))
+	    (setq results
+		  (nconc (nreverse (gnus-configure-posting-style
+				    (cdr (assoc v gnus-named-posting-styles))
+				    (cons v stack)))
+			 results)))
 	(push (cons element v) results)))
     (nreverse results)))
 
